@@ -227,42 +227,50 @@ gsd_log_default_handler (const gchar   *log_domain,
                                unused_data);
 }
 
-static gboolean
-daemonize (void)
+
+/* We want the parent process to quit after initializing all plugins,
+ * but we have to do all the work in the child process.  We can't
+ * initialize in parent and then fork here: that is not clean with
+ * X display and DBUS where we would make the connection from one
+ * process and continue using it from the other. So, we just make the
+ * parent to fork early and wait. */
+
+static void
+daemon_start (void)
 {
         int child_pid;
         char buf[1];
 
+        if (no_daemon)
+                return;
+
+        gnome_settings_profile_msg ("forking daemon");
+
         signal (SIGPIPE, SIG_IGN);
-        pipe (pipefds);
+        if (-1 == pipe (pipefds)) {
+                g_error ("Could not create pipe: %s", g_strerror (errno));
+                exit (EXIT_FAILURE);
+        }
+
         child_pid = fork ();
 
         switch (child_pid) {
         case -1:
-                return FALSE;
+                g_error ("Could not daemonize: %s", g_strerror (errno));
+                exit (EXIT_FAILURE);
 
         case 0:
                 /* child */
 
-                /* disconnect */
-                setsid ();
-                close (0);
-                close (1);
-                open ("/dev/null", O_RDONLY);
-                open ("/dev/null", O_WRONLY);
-
-                /* get outta the way */
-                chdir ("/");
-                umask (0117);
-
                 close (pipefds[0]);
 
-                return TRUE;
+                return;
 
          default:
                 /* parent */
 
                 close (pipefds[1]);
+
                 /* Wait for child to signal that we are good to go. */
                 read (pipefds[0], buf, 1);
 
@@ -270,6 +278,68 @@ daemonize (void)
         }
 }
 
+static void
+daemon_detach (void)
+{
+        if (no_daemon)
+                return;
+
+        gnome_settings_profile_msg ("detaching daemon");
+
+        /* disconnect */
+        setsid ();
+        close (0);
+        close (1);
+        open ("/dev/null", O_RDONLY);
+        open ("/dev/null", O_WRONLY);
+
+        /* get outta the way */
+        chdir ("/");
+        umask (0117);
+}
+
+static void
+daemon_terminate_parent (void)
+{
+        if (no_daemon)
+                return;
+
+        gnome_settings_profile_msg ("terminating parent");
+
+        write (pipefds[1], "1", 1);
+        close (pipefds[1]);
+}
+
+static void
+parse_args (int *argc, char ***argv)
+{
+        GError *error;
+        GOptionContext *context;
+
+        gnome_settings_profile_start (NULL);
+
+
+        context = g_option_context_new (NULL);
+
+        g_option_context_add_main_entries (context, entries, NULL);
+        g_option_context_add_group (context, gtk_get_option_group (FALSE));
+
+        error = NULL;
+        if (!g_option_context_parse (context, argc, argv, &error)) {
+                if (error != NULL) {
+                        g_warning ("%s", error->message);
+                        g_error_free (error);
+                } else {
+                        g_warning ("Unable to initialize GTK+");
+                }
+                exit (EXIT_FAILURE);
+        }
+
+        g_option_context_free (context);
+
+        gnome_settings_profile_end (NULL);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -285,32 +355,26 @@ main (int argc, char *argv[])
         bindtextdomain (GETTEXT_PACKAGE, GNOME_SETTINGS_LOCALEDIR);
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
         textdomain (GETTEXT_PACKAGE);
-
         setlocale (LC_ALL, "");
 
+        parse_args (&argc, &argv);
+
+        daemon_start ();
+
         g_type_init ();
 
-        gnome_settings_profile_start ("gtk init");
-        error = NULL;
-        if (! gtk_init_with_args (&argc, &argv, NULL, entries, NULL, &error)) {
-                if (error != NULL) {
-                        g_warning ("%s", error->message);
-                        g_error_free (error);
-                } else {
-                        g_warning ("Unable to initialize GTK+");
-                }
-                exit (1);
+        gnome_settings_profile_start ("opening gtk display");
+        if (! gtk_init_check (NULL, NULL)) {
+                g_warning ("Unable to initialize GTK+");
+                daemon_terminate_parent ();
+                exit (EXIT_FAILURE);
         }
-        gnome_settings_profile_end ("gtk init");
+        gnome_settings_profile_end ("opening gtk display");
+
+        daemon_detach ();
 
         g_log_set_default_handler (gsd_log_default_handler, NULL);
 
-        if (! no_daemon) {
-                gnome_settings_profile_start ("daemon initialization");
-                if (! daemonize ())
-                        g_error ("Could not daemonize: %s", g_strerror (errno));
-        }
-
         bus = get_session_bus ();
         if (bus == NULL) {
                 g_warning ("Could not get a connection to the bus");
@@ -347,19 +411,7 @@ main (int argc, char *argv[])
                 }
         }
 
-        /* Notify parent to exit.
-         *
-         * We want the parent process to quit after initializaing all plugins,
-         * but we have to do all the work in the child process.  We can't
-         * initialize in parent and then fork here: that is not clean with
-         * X display and DBUS where we would make the connection from one
-         * process and continue using from the other. So, we just made the
-         * parent to fork early and wait. */
-        if (! no_daemon) {
-                write (pipefds[1], "1", 1);
-                close (pipefds[1]);
-                gnome_settings_profile_end ("daemon initialization");
-        }
+        daemon_terminate_parent ();
 
         gtk_main ();
 
