@@ -71,15 +71,10 @@ struct GsdA11yKeyboardManagerPrivate
 #endif /* HAVE_LIBNOTIFY */
 };
 
-#define GSD_KBD_A11Y_ERROR gsd_kbd_a11y_error_quark ()
-
-enum {
-        GSD_KBD_A11Y_ERROR_NOT_AVAILABLE
-};
-
 static void     gsd_a11y_keyboard_manager_class_init  (GsdA11yKeyboardManagerClass *klass);
 static void     gsd_a11y_keyboard_manager_init        (GsdA11yKeyboardManager      *a11y_keyboard_manager);
 static void     gsd_a11y_keyboard_manager_finalize    (GObject             *object);
+static void     gsd_a11y_keyboard_manager_ensure_status_icon (GsdA11yKeyboardManager *manager);
 
 G_DEFINE_TYPE (GsdA11yKeyboardManager, gsd_a11y_keyboard_manager, G_TYPE_OBJECT)
 
@@ -92,19 +87,12 @@ static gpointer manager_object = NULL;
 #define d(str)          do { } while (0)
 #endif
 
-static GQuark
-gsd_kbd_a11y_error_quark (void)
-{
-        return g_quark_from_static_string ("gsd-kbd-a11y-error-quark");
-}
-
 static gboolean
 xkb_enabled (GsdA11yKeyboardManager *manager)
 {
         gboolean have_xkb;
         int opcode, errorBase, major, minor;
 
-        gdk_error_trap_push ();
         have_xkb = XkbQueryExtension (GDK_DISPLAY (),
                                       &opcode,
                                       &manager->priv->xkbEventBase,
@@ -112,8 +100,6 @@ xkb_enabled (GsdA11yKeyboardManager *manager)
                                       &major,
                                       &minor)
                 && XkbUseExtension (GDK_DISPLAY (), &major, &minor);
-        XSync (GDK_DISPLAY (), FALSE);
-        gdk_error_trap_pop ();
 
         return have_xkb;
 }
@@ -130,7 +116,6 @@ get_xkb_desc_rec (GsdA11yKeyboardManager *manager)
                 desc->ctrls = NULL;
                 status = XkbGetControls (GDK_DISPLAY (), XkbAllControlsMask, desc);
         }
-        XSync (GDK_DISPLAY (), FALSE);
         gdk_error_trap_pop ();
 
         g_return_val_if_fail (desc != NULL, NULL);
@@ -353,7 +338,7 @@ ax_response_callback (GsdA11yKeyboardManager *manager,
                       gboolean                enabled)
 {
         GConfClient *client;
-	GdkScreen *screen;
+        GdkScreen *screen;
         GError *err;
 
         switch (response_id) {
@@ -449,6 +434,10 @@ maybe_show_status_icon (GsdA11yKeyboardManager *manager)
         show = gconf_client_get_bool (client, CONFIG_ROOT "/enable", NULL);
         g_object_unref (client);
 
+        if (!show && manager->priv->status_icon == NULL)
+                return;
+
+        gsd_a11y_keyboard_manager_ensure_status_icon (manager);
         gtk_status_icon_set_visible (manager->priv->status_icon, show);
 }
 
@@ -538,7 +527,7 @@ ax_slowkeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
         message = _("You just held down the Shift key for 8 seconds.  This is the shortcut "
                     "for the Slow Keys feature, which affects the way your keyboard works.");
 
-        if (! gtk_status_icon_is_embedded (manager->priv->status_icon)) {
+        if (manager->priv->status_icon == NULL || ! gtk_status_icon_is_embedded (manager->priv->status_icon)) {
                 return FALSE;
         }
 
@@ -553,6 +542,7 @@ ax_slowkeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
                 manager->priv->notification = NULL;
         }
 
+        gsd_a11y_keyboard_manager_ensure_status_icon (manager);
         manager->priv->notification = notify_notification_new_with_status_icon (title,
                                                                                 message,
                                                                                 "preferences-desktop-accessibility",
@@ -681,7 +671,7 @@ ax_stickykeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
                 _("You just pressed two keys at once, or pressed the Shift key 5 times in a row.  "
                   "This turns off the Sticky Keys feature, which affects the way your keyboard works.");
 
-        if (! gtk_status_icon_is_embedded (manager->priv->status_icon)) {
+        if (manager->priv->status_icon == NULL || ! gtk_status_icon_is_embedded (manager->priv->status_icon)) {
                 return FALSE;
         }
 
@@ -696,6 +686,7 @@ ax_stickykeys_warning_post_bubble (GsdA11yKeyboardManager *manager,
                 manager->priv->notification = NULL;
         }
 
+        gsd_a11y_keyboard_manager_ensure_status_icon (manager);
         manager->priv->notification = notify_notification_new_with_status_icon (title,
                                                                                 message,
                                                                                 "preferences-desktop-accessibility",
@@ -1001,23 +992,17 @@ register_config_callback (GsdA11yKeyboardManager  *manager,
         *notify = gconf_client_notify_add (client, path, func, manager, NULL, NULL);
 }
 
-gboolean
-gsd_a11y_keyboard_manager_start (GsdA11yKeyboardManager *manager,
-                                 GError                **error)
+static gboolean
+start_a11y_keyboard_idle_cb (GsdA11yKeyboardManager *manager)
 {
         guint        event_mask;
         GConfClient *client;
-        gboolean     ret = FALSE;
 
         g_debug ("Starting a11y_keyboard manager");
         gnome_settings_profile_start (NULL);
 
-        if (!xkb_enabled (manager)) {
-                g_set_error (error, GSD_KBD_A11Y_ERROR,
-                             GSD_KBD_A11Y_ERROR_NOT_AVAILABLE,
-                             "XKB functionality is disabled.");
+        if (!xkb_enabled (manager))
                 goto out;
-        }
 
         client = gconf_client_get_default ();
 
@@ -1036,26 +1021,35 @@ gsd_a11y_keyboard_manager_start (GsdA11yKeyboardManager *manager,
         set_server_from_gconf (manager, client);
         g_object_unref (client);
 
-        gdk_error_trap_push ();
         XkbSelectEvents (GDK_DISPLAY (),
                          XkbUseCoreKbd,
                          event_mask,
                          event_mask);
 
-        XSync (GDK_DISPLAY (), FALSE);
-        gdk_error_trap_pop ();
-
         gdk_window_add_filter (NULL,
                                (GdkFilterFunc) cb_xkb_event_filter,
                                manager);
 
         maybe_show_status_icon (manager);
 
-        ret = TRUE;
-
  out:
         gnome_settings_profile_end (NULL);
-        return ret;
+
+        return FALSE;
+}
+
+
+gboolean
+gsd_a11y_keyboard_manager_start (GsdA11yKeyboardManager *manager,
+                                 GError                **error)
+{
+        gnome_settings_profile_start (NULL);
+
+        g_idle_add ((GSourceFunc) start_a11y_keyboard_idle_cb, manager);
+
+        gnome_settings_profile_end (NULL);
+
+        return TRUE;
 }
 
 void
@@ -1065,7 +1059,8 @@ gsd_a11y_keyboard_manager_stop (GsdA11yKeyboardManager *manager)
 
         g_debug ("Stopping a11y_keyboard manager");
 
-        gtk_status_icon_set_visible (manager->priv->status_icon, FALSE);
+        if (manager->priv->status_icon)
+                gtk_status_icon_set_visible (manager->priv->status_icon, FALSE);
 
         if (p->gconf_notify != 0) {
                 GConfClient *client = gconf_client_get_default ();
@@ -1198,17 +1193,28 @@ on_status_icon_activate (GtkStatusIcon          *status_icon,
         }
 }
 
+static void
+gsd_a11y_keyboard_manager_ensure_status_icon (GsdA11yKeyboardManager *manager)
+{
+        gnome_settings_profile_start (NULL);
+
+        if (!manager->priv->status_icon) {
+
+                manager->priv->status_icon = gtk_status_icon_new_from_icon_name ("preferences-desktop-accessibility");
+                g_signal_connect (manager->priv->status_icon,
+                                  "activate",
+                                  G_CALLBACK (on_status_icon_activate),
+                                  manager);
+        }
+
+        gnome_settings_profile_end (NULL);
+}
+
 static void
 gsd_a11y_keyboard_manager_init (GsdA11yKeyboardManager *manager)
 {
         manager->priv = GSD_A11Y_KEYBOARD_MANAGER_GET_PRIVATE (manager);
 
-        manager->priv->status_icon = gtk_status_icon_new_from_icon_name ("preferences-desktop-accessibility");
-        g_signal_connect (manager->priv->status_icon,
-                          "activate",
-                          G_CALLBACK (on_status_icon_activate),
-                          manager);
-
 #ifdef HAVE_LIBNOTIFY
         notify_init ("gnome-settings-daemon");
 #endif /* HAVE_LIBNOTIFY */
