@@ -39,7 +39,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <X11/keysym.h>
 
-#ifdef HAVE_XINPUT
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
 #include <X11/extensions/XInput.h>
 #endif
 #include <gconf/gconf.h>
@@ -55,17 +55,19 @@
 #define GCONF_MOUSE_DIR         "/desktop/gnome/peripherals/mouse"
 #define GCONF_MOUSE_A11Y_DIR    "/desktop/gnome/accessibility/mouse"
 
-#define KEY_LEFT_HANDED         "/desktop/gnome/peripherals/mouse/left_handed"
-#define KEY_MOTION_ACCELERATION "/desktop/gnome/peripherals/mouse/motion_acceleration"
-#define KEY_MOTION_THRESHOLD    "/desktop/gnome/peripherals/mouse/motion_threshold"
-#define KEY_LOCATE_POINTER      "/desktop/gnome/peripherals/mouse/locate_pointer"
-#define KEY_DWELL_ENABLE        "/desktop/gnome/accessibility/mouse/dwell_enable"
-#define KEY_DELAY_ENABLE        "/desktop/gnome/accessibility/mouse/delay_enable"
+#define KEY_LEFT_HANDED         GCONF_MOUSE_DIR "/left_handed"
+#define KEY_MOTION_ACCELERATION GCONF_MOUSE_DIR "/motion_acceleration"
+#define KEY_MOTION_THRESHOLD    GCONF_MOUSE_DIR "/motion_threshold"
+#define KEY_LOCATE_POINTER      GCONF_MOUSE_DIR "/locate_pointer"
+#define KEY_DWELL_ENABLE        GCONF_MOUSE_A11Y_DIR "/dwell_enable"
+#define KEY_DELAY_ENABLE        GCONF_MOUSE_A11Y_DIR "/delay_enable"
 
 struct GsdMouseManagerPrivate
 {
         guint notify;
         guint notify_a11y;
+
+        gboolean mousetweaks_daemon_running;
 };
 
 static void     gsd_mouse_manager_class_init  (GsdMouseManagerClass *klass);
@@ -153,7 +155,7 @@ gsd_mouse_manager_class_init (GsdMouseManagerClass *klass)
 }
 
 
-#ifdef HAVE_XINPUT
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
 static gboolean
 supports_xinput_devices (void)
 {
@@ -224,7 +226,7 @@ configure_button_layout (guchar   *buttons,
         }
 }
 
-#ifdef HAVE_XINPUT
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
 static gboolean
 xinput_device_has_buttons (XDeviceInfo *device_info)
 {
@@ -351,7 +353,7 @@ set_left_handed (GsdMouseManager *manager,
         gsize buttons_capacity = 16;
         gint n_buttons, i;
 
-#ifdef HAVE_XINPUT
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
         if (supports_xinput_devices ()) {
                 set_xinput_devices_left_handed (left_handed);
         }
@@ -583,9 +585,17 @@ set_mousetweaks_daemon (GsdMouseManager *manager,
 {
         GError *error = NULL;
         gchar *comm;
+        gboolean run_daemon = dwell_enable || delay_enable;
+
+        if (run_daemon || manager->priv->mousetweaks_daemon_running)
+                comm = g_strdup_printf ("mousetweaks %s",
+                                        run_daemon ? "" : "-s");
+        else
+                return;
+
+        if (run_daemon)
+                manager->priv->mousetweaks_daemon_running = TRUE;
 
-        comm = g_strdup_printf ("mousetweaks %s",
-                                (dwell_enable || delay_enable) ? "" : "-s");
 
         if (! g_spawn_command_line_async (comm, &error)) {
                 if (error->code == G_SPAWN_ERROR_NOENT &&
@@ -678,7 +688,7 @@ register_config_callback (GsdMouseManager         *manager,
                           const char              *path,
                           GConfClientNotifyFunc    func)
 {
-        gconf_client_add_dir (client, path, GCONF_CLIENT_PRELOAD_NONE, NULL);
+        gconf_client_add_dir (client, path, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
         return gconf_client_notify_add (client, path, func, manager, NULL, NULL);
 }
 
@@ -688,13 +698,11 @@ gsd_mouse_manager_init (GsdMouseManager *manager)
         manager->priv = GSD_MOUSE_MANAGER_GET_PRIVATE (manager);
 }
 
-gboolean
-gsd_mouse_manager_start (GsdMouseManager *manager,
-                         GError         **error)
+static gboolean
+gsd_mouse_manager_idle_cb (GsdMouseManager *manager)
 {
         GConfClient *client;
 
-        g_debug ("Starting mouse manager");
         gnome_settings_profile_start (NULL);
 
         client = gconf_client_get_default ();
@@ -710,7 +718,7 @@ gsd_mouse_manager_start (GsdMouseManager *manager,
                                           GCONF_MOUSE_A11Y_DIR,
                                           (GConfClientNotifyFunc) mouse_callback);
 
-#ifdef HAVE_XINPUT
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
         set_devicepresence_handler (manager);
 #endif
         set_mouse_settings (manager);
@@ -723,6 +731,19 @@ gsd_mouse_manager_start (GsdMouseManager *manager,
 
         gnome_settings_profile_end (NULL);
 
+        return FALSE;
+}
+
+gboolean
+gsd_mouse_manager_start (GsdMouseManager *manager,
+                         GError         **error)
+{
+        gnome_settings_profile_start (NULL);
+
+        g_idle_add ((GSourceFunc) gsd_mouse_manager_idle_cb, manager);
+
+        gnome_settings_profile_end (NULL);
+
         return TRUE;
 }
 
@@ -752,7 +773,7 @@ gsd_mouse_manager_stop (GsdMouseManager *manager)
 
         set_locate_pointer (manager, FALSE);
 
-#ifdef HAVE_XINPUT
+#ifdef HAVE_X11_EXTENSIONS_XINPUT_H
         gdk_window_remove_filter (NULL, devicepresence_filter, manager);
 #endif
 }
